#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <net/bpf.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <net/if.h>
#include <unistd.h>
#include "config.h"
#include "bpf.h"
#include "stats.h"
#include "iphandler.h"
#include "nitpicker.h"

/* 
 * Directly substitute libpcap with 
 * Berkeley Packet Filter
 * pcap is only able to handel 32k Buffersize.
 *
 */
#ifndef BPF_WORDALIGN
#define BPF_WORDALIGN(x) (((x)+(BPF_ALIGNMENT-1))&~(BPF_ALIGNMENT-1))
#endif


static int  fd;
static int bufsize;
static int kill=0;
static uint32 ts=0;
static char *bpfbuffer;
static struct bpf_stat bpfstats;


#ifdef PROF
  static int bpfread=0;
#endif


void bpf_readloop_terminate(int i) {
  kill = i;
}


uint16 bpf_datalink( void )
{
   uint16 dlink;
   ioctl( fd, BIOCGDLT, &dlink ); 

   return dlink;
}

void bpf_stats( int fd) { 

  if( ioctl( fd, BIOCGSTATS, &bpfstats ) < 0 ) {
     perror( "while getting bpfstats" );
     exit( 1 );
  }

  stats.total_pktcnt=bpfstats.bs_recv;
  stats.total_dropped=bpfstats.bs_drop;
}


void bpf_open( char *source )
{
 char device[sizeof "/dev/bpf0000000000"];
 struct ifreq ifr;
 struct timeval to;
 int v;
 int n=0;

 do {
  sprintf(device,"/dev/bpf%d", n++);
  fd = open(device, O_RDONLY);
 } while( fd < 0 && errno == EBUSY );

 if (fd < 0) { perror("Open bpf"); exit(1); }
 
 for( v = BPF_BUF; v != 0; v >>= 1 ) {
  ioctl(fd, BIOCSBLEN, &v);
  strcpy(ifr.ifr_name, source);
  if (ioctl(fd, BIOCSETIF, &ifr) >= 0) break;  
  if (errno != ENOBUFS) { perror("BIOCSBLEN"); exit(1); }
 }

 if( ioctl( fd, BIOCPROMISC, NULL ) ) {
   perror( "while setting to promisc mode" );
   exit( 1 );
 }

 if( ioctl(fd, BIOCGBLEN, &bufsize ) < 0) {
  perror( "while setting BIOCGBLEN" );
  exit( 1 );
 }

 bpfbuffer = (char *)calloc(1,bufsize);

 if( !bpfbuffer ) {
  perror( "Calloc Buffer" );
  exit(1);
 }

 /* set timeout */
 to.tv_sec = BPF_TIMEOUT ;
 to.tv_usec = 0 ;
 if (ioctl(fd, BIOCSRTIMEOUT,&to) < 0) { perror("Timeout.."); exit(1); };
}

int bpf_readloop()
{
  int buflen;
  int offset;
  int ph_len=0;
  int datalink;
  int oldts=0;
  struct timeval mytv;
  struct bpf_hdr *hd;
  void (*packethandler)(char *, uint32);

  datalink = bpf_datalink();  /* Here is no pcap handled yet.. FIX IT! XXX */

  switch( datalink ) {

    case DLT_EN10MB:
      ph_len = 0;		/* We don't know what kind of packet will come via Ethernet */
      packethandler = parse_etherpkt;  /* Create Functionpointer */
      break;

    case DLT_NULL:
      ph_len = 4;
      packethandler = iphandler_addPacket; /* Create another Functionpointer */ 
      break;
  }

  for (;;)
  {
    buflen=read(fd,bpfbuffer,bufsize);
    if (buflen<0){ perror("Read");exit(1); };   

#ifdef PROF
    bpfread++; /* time in seconds */
//    printf("%d\n",bpfread);
    if (bpfread > 500 ){  exit(0); };
#endif


    if (!buflen) 
    { 
       gettimeofday(&mytv,0);
       ts = mytv.tv_sec;

    }

    bpf_stats(fd);  
    offset=0;
    while ( offset < buflen ) {
      hd=(struct bpf_hdr*)(bpfbuffer+offset);
      ts=hd->bh_tstamp.tv_sec; /* Timestamp */

      (*packethandler)(bpfbuffer+ph_len+offset+hd->bh_hdrlen,ts);  /* Call funtionpointer */
      offset+=BPF_WORDALIGN(hd->bh_caplen+hd->bh_hdrlen);
    }
     
    if(  ts - oldts > 5 )  {
       oldts = ts;
       iphandler_check( ts );
    }
    if (is_stats_time(ts))  dump_stats();

    if ( kill ) {
      exit(0);
    }

  }
  return(0);
}

