#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include "config.h"
#include "version.h"

void config_init( struct config* cfg, int argc, char **argv )
{
    char opt;
    uint32 optcnt = 0;

    while (( opt = getopt(argc, argv, "d:i:I:f:s:")) != -1 ) {
        optcnt++;
        switch(opt) {

            case 'i': /* BPF */
                cfg->captureDevice = optarg;
                cfg->captureMethod = CM_BPF;   
                break;

            case 'I': /* PCAP Live */
                cfg->captureDevice = optarg;
                cfg->captureMethod = CM_PCAPLIVE; 
                break;

            case 'f': /* PCAP File */
                cfg->captureDevice = optarg;
                cfg->captureMethod = CM_PCAPFILE;   
                break;

            case 's':
                cfg->savedir = optarg;
                break;

            default:
                config_usage(argv);
        }
    }

    if( optcnt == 0 ) config_usage( argv );
}

void config_usage(char *argv[])      
{
 printf("Version:%s\n",Version);
 printf("Usage: %s [-i <interface>|-I <interface>|-f <pcapdumpfile>] [-d <debuglevel>] [-s <flowsavedir>]\n\n",*argv);
 printf("\t -i\tUses BPF Device to capture packets (recommended)\n");
 printf("\t -I\tUses libpcap to capture packets from a network device\n");
 printf("\t -f\tUses libpcap to read packets from a dumpfile (e.g. tcpdump)\n\n");
 exit(1);
}

