#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <pcap.h>
#include <bpf.h>
#include "config.h"
#include "iphandler.h"
#include "flowsaver.h"
#include "bpf.h"
#include "pcap.h"
#include "eth.h"
#include "stats.h"
#include "nitpicker.h"
#include "mysignal.h"
#include "stats.h"


static void (*readloop_terminate)(int);
struct config cfg;

void read_terminate(int kill) {
  dump_stats();
  (*readloop_terminate)(kill);
}

int main( int argc, char **argv )
{
    config_init( &cfg, argc, argv );

   /* init the handlers */
   signalhandler_init();
   iphandler_init();
   flowsaver_init( cfg.savedir );

   init_stats();
   opensource();

   iphandler_check( 0x7fffffff );
   flowsaver_saveflows(1);
   dump_stats();


   return(0);
}

int opensource()
{
  char *errbuf;
  pcap_t *pcap; 
  errbuf = NULL;

  switch ( cfg.captureMethod ) {

      case CM_BPF:
         readloop_terminate = bpf_readloop_terminate;
         bpf_open( cfg.captureDevice );
         bpf_readloop(); 
         break;;

      case CM_PCAPLIVE: 
         readloop_terminate = pcap_readloop_terminate;
         pcap = pcap_open_live( cfg.captureDevice, 256, 1, 1000, errbuf );
         pcap_readloop(pcap);
         break;

      case CM_PCAPFILE:
         readloop_terminate = pcap_readloop_terminate;
         pcap = pcap_open_offline( cfg.captureDevice, errbuf ); 
         pcap_readloop(pcap);
         break; 
  }

  if ( errbuf != NULL )
  {
    perror("Couldn't open source\n");
    exit(1);
  }

  return(0);
}
